/*
SD card demo

This is a sample demo project for Olimexino-Nano (Leonardo) board
for SD card based on the SD library example - CardInfo.

For Olimexino-Nano (Leonardo) hardware SPI pins are:
MOSI   - D16
MISO   - D14
SCK    - D15
SS     - D4 (this is the slave select of the hardware SPI)
PWREN  - D8  (output to low to enable UEXT)

After initializing the hardware in the main loop periodically
the program will scan for card, initialize it and show some info:
card type, files system, volume size and files and folders that contains.

Build and hardware info:
Olimexino-Nano rev.A
Arduino IDE v1.0.5

Stanimir Petev, Olimex LTD
2014.02.10
*/

// include the SD library:
#include <SD.h>

// set up variables using the SD utility library functions:
Sd2Card card;
SdVolume volume;
SdFile root;

// change this to match your SD shield or module;
// Arduino Ethernet shield: pin 4
// Adafruit SD shields and modules: pin 10
// Sparkfun SD shield: pin 8
// Olimexino-Nano (Leonardo): pin 4
const int chipSelect = 4;

void setup()
{
    // SPP +
    // UEXT power enable --> PORTB4 --> D8
    pinMode(8, OUTPUT);
    digitalWrite(8, LOW);

    // On the Ethernet Shield, CS is pin 4. It's set as an output by default.
    // Note that even if it's not used as the CS pin, the hardware SS pin 
    // (10 on most Arduino boards, 53 on the Mega) must be left as an output 
    // or the SD library functions will not work. 
    pinMode(4, OUTPUT); // SPP - Olimexino-Nano SPI's hardware function of SS is located at PORTB0 --> D4
    // SPP -
    
    // Open serial communications and wait for port to open:
    Serial.begin(9600);
    while (!Serial)
    {
        ; // wait for serial port to connect. Needed for Leonardo only
    }

}

void loop(void)
{
    Serial.print("\nInitializing SD card...");


    // we'll use the initialization code from the utility libraries
    // since we're just testing if the card is working!
    while (!card.init(SPI_HALF_SPEED, chipSelect))
    {
        Serial.println("initialization failed!\nThings to check:");
        Serial.println("* is a card is inserted?");
        Serial.println("* Is your wiring correct?");
        Serial.println("* did you change the chipSelect pin to match your shield or module?");
        Serial.println("\n\n\n");
        delay (4000);
        return;
    }

  
    Serial.println("Wiring is correct and a card is present.");
    // print the type of card
    Serial.print("\nCard type: ");
    switch (card.type())
    {
        case SD_CARD_TYPE_SD1:
            Serial.println("SD1");
            break;
        case SD_CARD_TYPE_SD2:
            Serial.println("SD2");
            break;
        case SD_CARD_TYPE_SDHC:
            Serial.println("SDHC");
            break;
        default:
            Serial.println("Unknown");
    }

    // Now we will try to open the 'volume'/'partition' - it should be FAT16 or FAT32
    if (!volume.init(card))
    {
        Serial.println("Could not find FAT16/FAT32 partition.\nMake sure you've formatted the card");
        Serial.println("\n\n\n");
        delay (4000);
        return;
    }


    // print the type and size of the first FAT-type volume
    uint32_t volumesize;
    Serial.print("\nVolume type is FAT");
    Serial.println(volume.fatType(), DEC);
    Serial.println();

    volumesize = volume.blocksPerCluster(); // clusters are collections of blocks
    volumesize *= volume.clusterCount(); // we'll have a lot of clusters
    
    #if 0    // old version if size of the card is less than 4 GB
      volumesize *= 512; // SD card blocks are always 512 bytes
      Serial.print("Volume size (bytes): ");
      Serial.println(volumesize);
      
      volumesize /= 1024;
    #else  // SPP in case of possible volume > 4 GB
      // If the card is bigger than 4 GB multiplication by 512 will
      // cause type overflow and so the volume value will be wrong
      // since the variable is only 32 bits but the value will be longer!
      // That's why instead of multiply by 512 and show bytes
      // it is divided by 2 and show the KBytes.
      volumesize /= 2;
    #endif
    Serial.print("Volume size (Kbytes): ");
    Serial.println(volumesize);
    
    volumesize /= 1024;
    Serial.print("Volume size (Mbytes): ");
    Serial.println(volumesize);


    Serial.println("\nFiles found on the card (name, date and size in bytes): ");
    root.openRoot(volume);

    // list all files in the card with date and size
    root.ls(LS_R | LS_DATE | LS_SIZE);
    Serial.println("\n\n\n");
    delay (4000);
}
